﻿unit TestVSoftRegularExpression;
{

  Delphi DUnit Test Case
  ----------------------
  This unit contains a skeleton test case class generated by the Test Case Wizard.
  Modify the generated code to correctly setup and call the methods from the unit
  being tested.

}

interface

uses
  TestFramework,
  classes,
  TypInfo,
  VSoft.RegularExpression;

type
  TVSoftRegularExpressionTests = class(TTestCase)
  private
    regExpr: IRegularExpression;
  public
    procedure SetUp; override;
  published
    procedure TestReplace;
    procedure TestReplaceAll;
    procedure TestExecute;
    procedure TestMultiLine;
    procedure TestSingleLine;
    procedure TestCaseSensitive;
    procedure TestMatches;
    procedure TestMatchAgain;
    procedure TestGroups;
    //procedure TestMatchResult;
    procedure TestCompiled;
  end;

implementation
uses
  Windows,
  SysUtils;

  { TestRegExprUtils }

procedure TVSoftRegularExpressionTests.SetUp;
begin
  inherited;
end;

procedure TVSoftRegularExpressionTests.TestCaseSensitive;
var
  sTest: string;
begin
  regExpr := RegEx.NewRegularExpression('string', [RegExOption.IgnoreCase]);
  sTest := 'Test STRING';
  Check(regExpr.IsMatch(sTest),'Execute failed on not case sensitive');

  regExpr := nil;
  regExpr := RegEx.NewRegularExpression('string');
  Check(not regExpr.IsMatch(sTest),'Execute failed on lowercase');

  regExpr := nil;
  regExpr := RegEx.NewRegularExpression('STRING');
  Check(regExpr.IsMatch(sTest),'Execute failed on uppercase');
end;

procedure TVSoftRegularExpressionTests.TestCompiled;
begin
  try
    regExpr := RegEx.NewRegularExpression('(\d+)\.(\d+)\.?(\d+)?\.?(\d+)?', [RegExOption.Compiled]);
    Check(True, 'Compile of valid expression succeeded');
  except
    Check(False, 'Compile of valid expression failed');
  end;
  regExpr := nil;

  try
    regExpr := RegEx.NewRegularExpression('??$^*&(', [RegExOption.Compiled]);
    Check(False, 'Compile of valid expression failed');
  except
    Check(True, 'Compile of invalid expression succeeded');
  end;
end;

procedure TVSoftRegularExpressionTests.TestExecute;
begin
  regExpr := RegEx.NewRegularExpression('[A-Za-z]');
  Check(regExpr.IsMatch('Test string'),'Execute failed on alpha');
  Check(not regExpr.IsMatch('123?<>[]'),'Execute failed on non alpha');
  regExpr := nil;
  regExpr := RegEx.NewRegularExpression('世');
  Check(regExpr.IsMatch('世界您好'),'Execute failed on unicode');
end;

procedure TVSoftRegularExpressionTests.TestGroups;
var
  sVersion: string;
  aMatch: IMatch;
begin
  regExpr := RegEx.NewRegularExpression('(\d+)\.(\d+)\.?(\d+)?\.?(\d+)?');
  sVersion := '7.12.250.1000';
  aMatch := regExpr.Match(sVersion);
  Check(aMatch.Groups.Count = 5, 'Count should be 5, is ' + IntToStr(aMatch.Groups.Count));
  Check(aMatch.Groups.Item[1].Value = '7', 'major version should be "7"');
  Check(aMatch.Groups.Item[2].Value = '12', 'major version should be "12"');
  Check(aMatch.Groups.Item[3].Value = '250', 'major version should be "250"');
  Check(aMatch.Groups.Item[4].Value = '1000', 'major version should be "1000"');
end;

procedure TVSoftRegularExpressionTests.TestMatchAgain;
var
  sTest : string;
  aMatch: IMatch;
begin
 regExpr := RegEx.NewRegularExpression('^(mapped)(Version):\s+(.*)$', [RegExOption.MultiLine]);
 sTest := 'Exiting the getSyntax subroutine...' + #13#10 +
          'mappedVersion:  3.50.00' + #13#10 +
          'Entering the getMappedBuildVersion subroutine...' + #13#10 +
          'Exiting the getMappedBuildVersion subroutine...' + #13#10 +
           #13#10 +
          'mappedVersion:  5.50.00';

 Check(regExpr.IsMatch(sTest), 'Initial Match Failed, should have found 2 matches');
 Check(regExpr.Matches(sTest).Count = 2, 'Match count should be 2');
 aMatch := regExpr.Match(sTest);
 Check(aMatch.Success, 'First match object failed');
 Check(aMatch.Groups.Item[3].Value = '3.50.00', 'First match object''s group 3 should be 3.50.00, but is ' + aMatch.Groups.Item[3].Value);

 aMatch := aMatch.NextMatch;
 Check(aMatch.Success, 'Second match object failed');
 Check(aMatch.Groups.Item[3].Value = '5.50.00', 'Second match object''s group 3 should be 5.50.00, but is ' + aMatch.Groups.Item[3].Value);
end;

procedure TVSoftRegularExpressionTests.TestMatches;
var
  sVersion: string;
  cResults: IMatchCollection;
begin
  regExpr := RegEx.NewRegularExpression('(\d+)\.(\d+)\.?(\d+)?\.?(\d+)?');
  sVersion := '7.12.250.1000    23.3.4.3';
  cResults := regExpr.Matches(sVersion);
  Check(cResults.Count = 2, 'Count should be 2, is ' + IntToStr(cResults.Count));
  Check(cResults.Match[0].Value = '7.12.250.1000', 'expected 7.12.250.1000, result returned is ' + cResults.Match[0].Value);
  Check(cResults.Match[1].Value = '23.3.4.3','expected 23.3.4.3, result returned is ' + cResults.Match[1].Value);
end;

(*procedure TVSoftRegularExpressionTests.TestMatchResult;
var
  sInput: string;
  aMatch: IMatch;
  sResult: string;
begin
  regExpr := RegEx.NewRegularExpression('\w+');
  sInput  := 'here is a collection of words in a string';
  aMatch  := regExpr.Match(sInput);
  sResult := aMatch.Value;
  sResult := aMatch.Result('The first word is "$&"');
  Check(sResult = 'The first word is "here"');
end;*)

procedure TVSoftRegularExpressionTests.TestMultiLine;
var
  sTest: string;
begin
  regExpr := RegEx.NewRegularExpression('Test\s.*string', [RegExOption.MultiLine]);
  sTest := 'Test string';
  Check(regExpr.IsMatch(sTest),'Execute failed on single line input with multiline option');
  sTest := 'Test ' + #13#10 + 'string';
  Check(not regExpr.IsMatch(sTest),'Execute failed on multi line input with multiline option');
  regExpr := nil;

  regExpr := RegEx.NewRegularExpression('^Test\s$\r\n^string$', [RegExOption.MultiLine]);
  sTest := 'Test string';
  Check(not regExpr.IsMatch(sTest),'Execute failed on single line input with multiline option');
  sTest := 'Test ' + #13#10 + 'string';
  Check(regExpr.IsMatch(sTest),'Execute failed on multi line input with multiline option, using anchors, should have failed');
  regExpr := nil;

  regExpr := RegEx.NewRegularExpression('^Test\s.*string$', [RegExOption.MultiLine]);
  sTest := 'Test string';
  Check(regExpr.IsMatch(sTest),'Execute failed on single line input with multiline option');
  sTest := 'Test ' + #13#10 + 'string';
  Check(not regExpr.IsMatch(sTest),'Execute failed on multi line input with multiline option, using anchors, should have failed');
  regExpr := nil;

  regExpr := RegEx.NewRegularExpression('Test\s.*string');
  sTest := 'Test string';
  Check(regExpr.IsMatch(sTest),'Execute failed on single line');
  sTest := 'Test ' + #13#10 + 'string';
  Check(not regExpr.IsMatch(sTest),'Execute passed on multiline without the multiline option, should have failed');
// DW does setting none clear the flags???

  regExpr := RegEx.NewRegularExpression('^Test\s\r\nstring$', [RegExOption.None]);
  sTest := 'Test string';
  Check(not regExpr.IsMatch(sTest),'Execute failed on single line, no carriage return');
  sTest := 'Test ' + #13#10 + 'string';
  Check(regExpr.IsMatch(sTest),'Execute passed on multiline without the multiline option, using anchors, should have passed');
end;

procedure TVSoftRegularExpressionTests.TestSingleLine;
var
  sTest: string;
begin
  regExpr := RegEx.NewRegularExpression('Test\s.*string', [RegExOption.SingleLine]);
  sTest := 'Test string';
  Check(regExpr.IsMatch(sTest),'Execute failed on single line input with multiline option');
  sTest := 'Test ' + #13#10 + 'string';
  Check(regExpr.IsMatch(sTest),'Execute failed on multi line input with multiline option');
  regExpr := nil;

  regExpr := RegEx.NewRegularExpression('Test\s.*string');
  sTest := 'Test string';
  Check(regExpr.IsMatch(sTest),'Execute failed on single line');
  sTest := 'Test ' + #13#10 + 'string';
  Check(not regExpr.IsMatch(sTest),'Execute passed on multiline without the multiline option, should have failed');
end;

procedure TVSoftRegularExpressionTests.TestReplaceAll;
var
  sInput: string;
  sExpResult: string;
  sActResult: string;
begin
  regExpr := RegEx.NewRegularExpression('(Monday)|(Tueday)|(Wedday)|(Thursday)|(Friday)|(Satday)|(Sunday)|(Mon)|(Tue)|(Wed)|(Thu)|(Fri)|(Sat)|(Sun)');
  sInput := 'This is a string with days in it Monday Sunday Thu';
  sExpResult := 'This is a string with days in it   ';
  sActResult := regExpr.Replace(sInput, '');
  Check(sActResult = sExpResult, 'New reg expr "replace" failed, result = ' + sActResult);
  regExpr := nil;

  regExpr := RegEx.NewRegularExpression('世');
  sInput := '世界您好';
  sExpResult := '界您好';
  sActResult := regExpr.Replace(sInput, '');
  Check(sActResult = sExpResult, 'New reg expr unicode "replace" failed, result = ' + sActResult);
end;

procedure TVSoftRegularExpressionTests.TestReplace;
var
  sInput: string;
  sExpResult: string;
  sActResult: string;
begin
  regExpr := RegEx.NewRegularExpression('we');
  sInput := 'we have been working on this. we want to replace we with us';
  sExpResult := 'us have been working on this. us want to replace us with us';
  sActResult := regExpr.Replace(sInput, 'us', 3);
  Check(sActResult = sExpResult, 'New reg expr "replace, 3" failed, result = ' + sActResult);

  sExpResult := 'us have been working on this. us want to replace we with us';
  sActResult := regExpr.Replace(sInput, 'us', 2);
  Check(sActResult = sExpResult, 'New reg expr "replace, 2" failed, result = ' + sActResult);
end;

initialization
  // Register any test cases with the test runner
  RegisterTest(TVSoftRegularExpressionTests.Suite);

end.

